part of tecfy_core_client_package;

class BarChartSample2 extends StatefulWidget {
  final DashboardItemBase type;
  BarChartSample2(this.type);
  @override
  State<StatefulWidget> createState() => _BarChartSample2State();
}

class _BarChartSample2State extends State<BarChartSample2> {
  final Color _leftBarColor = const Color(0xff53fdd7);
  final Color _rightBarColor = const Color(0xffff5182);
  final double width = 7;

  late List<BarChartGroupData> _rawBarGroups;
  late List<BarChartGroupData> _showingBarGroups;

  int? _touchedGroupIndex;

  @override
  void initState() {
    super.initState();
    final barGroup1 = _makeGroupData(0, 5, 12);
    final barGroup2 = _makeGroupData(1, 16, 12);
    final barGroup3 = _makeGroupData(2, 18, 5);
    final barGroup4 = _makeGroupData(3, 20, 16);
    final barGroup5 = _makeGroupData(4, 17, 6);
    final barGroup6 = _makeGroupData(5, 19, 1.5);
    final barGroup7 = _makeGroupData(6, 10, 1.5);

    final items = [
      barGroup1,
      barGroup2,
      barGroup3,
      barGroup4,
      barGroup5,
      barGroup6,
      barGroup7,
    ];

    _rawBarGroups = items;

    _showingBarGroups = _rawBarGroups;
  }

  @override
  Widget build(BuildContext context) {
    return AspectRatio(
      aspectRatio: 1,
      child: Card(
        elevation: 1,
        shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(4)),
        //color: const Color(0xff2c4260),
        child: Padding(
          padding: const EdgeInsets.all(16),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.stretch,
            mainAxisAlignment: MainAxisAlignment.start,
            mainAxisSize: MainAxisSize.max,
            children: <Widget>[
              Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.start,
                children: <Widget>[
                  Icon(Icons.people, color: Colors.black54),
                  //  makeTransactionsIcon(),
                  const SizedBox(width: 20),
                  Text(
                    widget.type.title,
                    style: TextStyle(color: Colors.black38, fontSize: 22),
                  ),
                ],
              ),
              const SizedBox(height: 38),
              Expanded(
                child: Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 8.0),
                  child: BarChart(
                    BarChartData(
                      maxY: 20,
                      barTouchData: BarTouchData(
                          touchTooltipData: BarTouchTooltipData(
                            tooltipBgColor: Colors.grey,
                            getTooltipItem: (_a, _b, _c, _d) => null,
                          ),
                          touchCallback: (t, response) {
                            if (response?.spot == null) {
                              setState(() {
                                _touchedGroupIndex = -1;
                                _showingBarGroups = List.of(_rawBarGroups);
                              });
                              return;
                            }

                            _touchedGroupIndex =
                                response!.spot!.touchedBarGroupIndex;

                            setState(() {
                              // if (response.touchInput is FlLongPressEnd ||
                              //     response.touchInput is FlPanEnd) {
                              //   touchedGroupIndex = -1;
                              //   showingBarGroups = List.of(rawBarGroups);
                              // } else
                              {
                                _showingBarGroups = List.of(_rawBarGroups);
                                if (_touchedGroupIndex != -1) {
                                  double sum = 0;
                                  for (BarChartRodData rod
                                      in _showingBarGroups[_touchedGroupIndex!]
                                          .barRods) {
                                    sum += rod.toY;
                                  }
                                  final avg = sum /
                                      _showingBarGroups[_touchedGroupIndex!]
                                          .barRods
                                          .length;

                                  _showingBarGroups[_touchedGroupIndex!] =
                                      _showingBarGroups[_touchedGroupIndex!]
                                          .copyWith(
                                    barRods:
                                        _showingBarGroups[_touchedGroupIndex!]
                                            .barRods
                                            .map((rod) {
                                      return rod.copyWith(toY: avg);
                                    }).toList(),
                                  );
                                }
                              }
                            });
                          }),
                      titlesData: FlTitlesData(
                        show: true,
                        bottomTitles: AxisTitles(
                            sideTitles: SideTitles(
                          showTitles: true,

                          // getTitlesWidget: (c, d) => TextStyle(
                          //     color: const Color(0xff7589a2),
                          //     fontWeight: FontWeight.bold,
                          //     fontSize: 14),
                          //margin: 20,
                          getTitlesWidget: (double value, d) {
                            switch (value.toInt()) {
                              case 0:
                                return Text('Mn');
                              case 1:
                                return Text('Te');
                              case 2:
                                return Text('Wd');
                              case 3:
                                return Text('Tu');
                              case 4:
                                return Text('Fr');
                              case 5:
                                return Text('St');
                              case 6:
                                return Text('Sn');
                              default:
                                return Text('');
                            }
                          },
                        )),
                        leftTitles: AxisTitles(
                            sideTitles: SideTitles(
                          showTitles: true,
                          // getTextStyles: (c, d) => TextStyle(
                          //     color: const Color(0xff7589a2),
                          //     fontWeight: FontWeight.bold,
                          //     fontSize: 14),
                          // margin: 32,
                          reservedSize: 14,
                          getTitlesWidget: (value, c) {
                            if (value == 0) {
                              return Text('1K');
                            } else if (value == 10) {
                              return Text('5K');
                            } else if (value == 19) {
                              return Text('10K');
                            } else {
                              return Text('');
                            }
                          },
                        )),
                      ),
                      borderData: FlBorderData(
                        show: false,
                      ),
                      barGroups: _showingBarGroups,
                    ),
                  ),
                ),
              ),
              const SizedBox(
                height: 12,
              ),
            ],
          ),
        ),
      ),
    );
  }

  BarChartGroupData _makeGroupData(int x, double y1, double y2) {
    return BarChartGroupData(barsSpace: 4, x: x, barRods: [
      BarChartRodData(
        toY: y1,
        color: _leftBarColor,
        width: width,
      ),
      BarChartRodData(
        toY: y2,
        color: _rightBarColor,
        width: width,
      ),
    ]);
  }

  Widget _makeTransactionsIcon() {
    const double width = 4.5;
    const double space = 3.5;
    return Row(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Container(
          width: width,
          height: 10,
          color: Colors.black.withOpacity(0.4),
        ),
        const SizedBox(
          width: space,
        ),
        Container(
          width: width,
          height: 28,
          color: Colors.black.withOpacity(0.8),
        ),
        const SizedBox(
          width: space,
        ),
        Container(
          width: width,
          height: 42,
          color: Colors.black.withOpacity(1),
        ),
        const SizedBox(
          width: space,
        ),
        Container(
          width: width,
          height: 28,
          color: Colors.black.withOpacity(0.8),
        ),
        const SizedBox(
          width: space,
        ),
        Container(
          width: width,
          height: 10,
          color: Colors.black.withOpacity(0.4),
        ),
      ],
    );
  }
}
