part of tecfy_core_client_package;

class PorpertyComponent extends StatefulWidget {
  /// the property object
  final Property property;

  /// the main entity
  final Entity? entity;

  /// change the value style
  final TextStyle? valueStyle;

  /// if we don't need to show label
  final bool noLabel;

  /// is add new so we can show or hide according to property config
  final bool isAddNew;

  /// on change
  final void Function(dynamic)? onChange;

  /// on submit
  final void Function(dynamic)? onSubmit;

  /// on update
  final void Function(Map<String, dynamic>)? onUpdateData;
  //final dynamic initalValue;
  /// if this property is a dropdown and it's filterd by another field
  //final dynamic filterValue;
  final Map<String, dynamic> data;

  /// show all property as read only
  final bool readOnly;

  ///isFoucsed
  //final bool isFoucs;

  /// on open Popup
  final void Function()? onOpen;

  /// on close popup
  final void Function()? onClose;
  PorpertyComponent({
    required this.property,
    this.entity,
    this.onChange,
    this.onSubmit,
    this.isAddNew = false,
    this.onUpdateData,
    this.valueStyle,
    this.noLabel = false,
    //this.filterValue,
    required this.data,
    this.readOnly = false,
    this.onOpen,
    this.onClose,
  }); //: assert(entity != null || property.isRequired ==
  // false);
  @override
  _PorpertyComponentState createState() => _PorpertyComponentState();
}

class _PorpertyComponentState extends State<PorpertyComponent> {
  TextEditingController? _textController;
  FocusNode? _focusNode;
  dynamic _initValue;
  //bool isVisable =true;
  bool _isVisible = true;
  @override
  void initState() {
    _init();
    super.initState();
  }

  void _init() {
    _initValue = widget.data[widget.property.dbName];
    var val = _initValue?.toString();
    if (widget.property.propertyType == PropertyTypes.Number && val != null) {
      if (val.contains('.'))
        val = (_initValue as num).toDouble().toStringAsFixed(2);
    }
    if (widget.property.propertyType == PropertyTypes.Text ||
        widget.property.propertyType == PropertyTypes.Number) {
      if (_textController == null) {
        _textController = TextEditingController(text: val ?? '');
        _focusNode = FocusNode();
        _textController!.addListener(() {
          _submitChages();
        });
      } else {
        _textController?.text = val ?? '';
      }
    }
  }

  void _submitChages() {
    if (widget.property.propertyType == PropertyTypes.Text) {
      _initValue = _textController?.text;
      if (widget.onChange != null) widget.onChange?.call(_textController?.text);
    }
    if (widget.property.propertyType == PropertyTypes.Number) {
      _initValue = num.tryParse(_textController?.text ?? '');
      if (widget.onChange != null) widget.onChange?.call(_initValue);
    }
  }

  // _getInitValueString() {
  //   return _initValue == null
  //       ? '__'
  //       : _initValue is int
  //           ? _initValue.toString()
  //           : _initValue is num
  //               ? (_initValue as num).toDouble().toStringAsFixed(2)
  //               : _initValue.toString();
  // }

  @override
  Widget build(BuildContext context) {
    widget.property.isVisible(widget.data, widget.entity).then((isVisible) {
      if (isVisible != _isVisible) {
        _isVisible = isVisible;
        setState(() {});
      }
    });

    if (!_isVisible) return const SizedBox();

    //if (!widget.property.isVisable) return Container();
    // print(
    //     '-------------------------- ${widget.property.dbName} --- ${widget.property.readOnly}');
    if (_initValue != widget.data[widget.property.dbName]) _init();
    if (widget.readOnly) return _getReadOnlyWidget();

    if (widget.property.readOnly &&
        widget.property.propertyType != PropertyTypes.Dropdown &&
        widget.property.propertyType != PropertyTypes.OtherEntityMany &&
        widget.property.propertyType != PropertyTypes.ManyToMany &&
        widget.property.propertyType != PropertyTypes.OneToMany) {
      if (widget.noLabel)
        return Text(widget.property.getDisplayText(widget.data));
      else
        return TextFormLabel(
          label: widget.property.name,
          value: widget.property.getDisplayText(widget.data),
        );
    }

    switch (widget.property.propertyType) {
      case PropertyTypes.Dropdown:
        var prop = (widget.property as PropertyDropdown);
        if (prop.options != null) {
          return DropdownComponenet(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            options: prop.options,
            readOnly: widget.property.readOnly,
            displayProperty: LocalConfigService.isArabic ? 'nameAr' : 'nameEn',
            idProperty: 'id',
            initalValue: widget.data[prop.dbName],
            onChanged: (v) {
              widget.data[prop.dbName] = v;
              widget.onChange?.call(v);
              widget.onUpdateData?.call(widget.data);
              widget.data[prop.dbName] = v;
            },
          );
        }
        return DropdownAdvancedEntity(prop.entityName,
            onOpen: widget.onOpen,
            onClose: widget.onClose,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            entity: EntityService.getEntity(prop.entityName)!,
            url: prop.dropdownUrl,
            property: prop,
            readOnly: widget.property.readOnly,
            data: widget.data, onChanged: (v, data) {
          widget.onChange?.call(v);
          if (data != null && widget.onUpdateData != null)
            widget.onUpdateData?.call(widget.data);
        });
      case PropertyTypes.OtherEntityMany:
        return ListFormProperty(widget.isAddNew,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            readOnly: widget.property.readOnly,
            entityId: (widget.property as PropertyOtherEntityMany).entityName,
            data: _initValue == null
                ? []
                : _initValue is String
                    ? []
                    : _initValue, onChanged: (v) {
          widget.onChange?.call(v);
        });

      case PropertyTypes.ManyToMany:
        if (!(widget.property as PropertyManyToMany).isWithData) {
          return ManyToManyProperty(
            label: widget.noLabel ? null : widget.property.name,
            // description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            selectedValues: _initValue,
            entity: EntityService.getEntity(
                (widget.property as PropertyManyToMany).entityName)!,
            onChanged: (v) => widget.onChange?.call(v),
          );
        }

        return ManyToManyWithData(
          (widget.property as PropertyManyToMany).entityName,
          label: widget.noLabel ? null : widget.property.name,
          // description: widget.property.description,
          isRequired: widget.noLabel ? false : widget.property.isRequired,
          items: _initValue,
          entity: EntityService.getEntity(
              (widget.property as PropertyManyToMany).entityName)!,
          onChanged: (v) => widget.onChange?.call(v),
          entityMiddle:
              (widget.property as PropertyManyToMany).middleEntityName,
          propertieMiddle:
              (widget.property as PropertyManyToMany).middleOtherPropertyName,
        );

      case PropertyTypes.OneToMany:
        return ManyToManyProperty(
          label: widget.noLabel ? null : widget.property.name,
          // description: widget.property.description,
          isRequired: widget.noLabel ? false : widget.property.isRequired,
          selectedValues: _initValue,
          entity: EntityService.getEntity(
              (widget.property as PropertyOneToMany).entityName)!,
          onChanged: (v) => widget.onChange?.call(v),
        );

      case PropertyTypes.Text:
        if ((widget.property as PropertyText).isBarcode) {
          return BarcodePropertyComponenet(
            labelText: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            textStyle: widget.valueStyle,
            focusNode: _focusNode,
            controller: _textController,
            onFieldSubmitted: (v) {
              _focusNode?.unfocus();
              widget.onSubmit?.call(_initValue);
            },
            onEditingComplete: _submitChages,
          );
        }
        return AppTextFormField(
            textDirection: LocalConfigService.isArabic &&
                    !(widget.property as PropertyText).isEmail
                ? TextDirection.rtl
                : TextDirection.ltr,
            labelText: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            textStyle: widget.valueStyle,
            focusNode: _focusNode,
            controller: _textController,
            onFieldSubmitted: (v) {
              _focusNode?.unfocus();
              widget.onSubmit?.call(_initValue);
            },
            textInputAction: TextInputAction.next,
            obscureText: (widget.property as PropertyText).isPassword,
            keyboardType: (widget.property as PropertyText).isEmail
                ? TextInputType.emailAddress
                : (widget.property as PropertyText).lines > 1
                    ? TextInputType.multiline
                    : TextInputType.text,
            maxLines: (widget.property as PropertyText).lines,
            onEditingComplete: _submitChages);
      case PropertyTypes.Number:
        return AppTextFormField(
            labelText: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            controller: _textController,
            focusNode: _focusNode,
            onFieldSubmitted: (v) => _focusNode?.unfocus(),
            textStyle: widget.valueStyle,
            textInputAction: TextInputAction.next,
            keyboardType: TextInputType.number,
            onEditingComplete: _submitChages);
      case PropertyTypes.Boolean:
        return AppCheckBox(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            initalValue: _initValue,
            isFullrow: true,
            onChanged: (v) {
              if (v == _initValue) return;
              widget.onChange?.call(v);
            });
      case PropertyTypes.File:
        return PhotoProperty(
            fileData: _initValue,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            fileId: widget.data[widget.property.dbName + 'Id'],
            maxSize: (widget.property as PropertyFile).maxSize,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.Color:
        return ColorPropertyComponenet(
            initalValue: _initValue?.toString(),
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.MapArea:
        return MapAreaPropertyComponenet(
            initalValue: _initValue,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.MarkImage:
        return MarkImagePropertyComponenet(
            initalValue: _initValue,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.Date:
        return DateProperty(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.DateTime:
        return DateTimeProperty(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.Time:
        return TimeProperty(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.Location:
        return LocationPropertyComponenet(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.MapArea:
        return MapAreaPropertyComponenet(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      default:
        return Text(
            '${widget.property.dbName} ${widget.property.propertyType}');
    }
  }

  Widget _getReadOnlyWidget() {
    switch (widget.property.propertyType) {
      case PropertyTypes.Dropdown:
        var property = (widget.property as PropertyDropdown);
        var entity = EntityService.getEntity(property.entityName);
        String val;
        if (property.options != null) {
          val = widget.data[property.dbName] ?? '-';
        } else if (widget.data[property.dbNameEntity] == null) {
          EntityService.getOne(
                  entity!.dbName, widget.data[widget.property.dbName])
              .then((result) {
            if (result != null) {
              setState(() {
                widget.data[property.dbNameEntity] = result;
              });
            }
          });
          val = '__';
        } else {
          val = widget.data[property.dbNameEntity][entity!.displayField.dbName]
                  ?.toString() ??
              widget.data[property.dbNameEntity]['display'] ??
              widget.data[property.dbNameEntity]['name'] ??
              widget.data[property.dbNameEntity]['nameAr'] ??
              widget.data[property.dbNameEntity]['nameEn'] ??
              widget.data[property.dbNameEntity]['title'] ??
              '__';
        }
        if (widget.noLabel)
          return Text(val, style: widget.valueStyle);
        else
          return TextFormLabel(
            label: widget.property.name,
            description: widget.property.description,
            valueStyle: widget.valueStyle,
            value: val,
          );

      case PropertyTypes.OtherEntityMany:
        return ListFormProperty(widget.isAddNew,
            readOnly: true,
            label: widget.noLabel ? null : widget.property.name,
            entityId: (widget.property as PropertyOtherEntityMany).entityName,
            data: _initValue ?? [],
            onChanged: (v) {});
      default:
        if (widget.noLabel)
          return Text(widget.property.getDisplayText(widget.data),
              style: widget.valueStyle);
        try {
          return TextFormWidget(
            label: widget.noLabel ? null : widget.property.name,
            value: widget.property
                .getDisplayWidget(widget.data, textStyle: widget.valueStyle),
          );
        } catch (err) {
          return TextFormLabel(
              label: widget.property.name,
              value: widget.property.getDisplayText(widget.data),
              valueStyle: widget.valueStyle);
        }
    }
  }
}
